package sun.font;

import java.awt.geom.GeneralPath;
import java.awt.geom.Area;
import java.awt.Color;
import java.awt.Stroke;
import java.awt.geom.Line2D;
import java.awt.BasicStroke;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.awt.Graphics2D;
import java.util.Map;

public class Decoration
{
    private static final int VALUES_MASK;
    private static final Decoration PLAIN;
    
    private Decoration() {
    }
    
    public static Decoration getPlainDecoration() {
        return Decoration.PLAIN;
    }
    
    public static Decoration getDecoration(AttributeValues applyIMHighlight) {
        if (applyIMHighlight == null || !applyIMHighlight.anyDefined(Decoration.VALUES_MASK)) {
            return Decoration.PLAIN;
        }
        applyIMHighlight = applyIMHighlight.applyIMHighlight();
        return new Decoration.DecorationImpl(applyIMHighlight.getForeground(), applyIMHighlight.getBackground(), applyIMHighlight.getSwapColors(), applyIMHighlight.getStrikethrough(), Underline.getUnderline(applyIMHighlight.getUnderline()), Underline.getUnderline(applyIMHighlight.getInputMethodUnderline()));
    }
    
    public static Decoration getDecoration(Map map) {
        if (map == null) {
            return Decoration.PLAIN;
        }
        return getDecoration(AttributeValues.fromMap(map));
    }
    
    public void drawTextAndDecorations(Decoration.Label label, Graphics2D graphics2D, float n, float n2) {
        label.handleDraw(graphics2D, n, n2);
    }
    
    public Rectangle2D getVisualBounds(Decoration.Label label) {
        return label.handleGetVisualBounds();
    }
    
    public Rectangle2D getCharVisualBounds(Decoration.Label label, int n) {
        return label.handleGetCharVisualBounds(n);
    }
    
    Shape getOutline(Decoration.Label label, float n, float n2) {
        return label.handleGetOutline(n, n2);
    }
    
    Decoration(Decoration.1 object) {
        this();
    }
    
    static {
        VALUES_MASK = AttributeValues.getMask(new EAttribute[] { EAttribute.EFOREGROUND, EAttribute.EBACKGROUND, EAttribute.ESWAP_COLORS, EAttribute.ESTRIKETHROUGH, EAttribute.EUNDERLINE, EAttribute.EINPUT_METHOD_HIGHLIGHT, EAttribute.EINPUT_METHOD_UNDERLINE });
        PLAIN = new Decoration();
    }
    
    private static final class DecorationImpl extends Decoration
    {
        private Paint fgPaint;
        private Paint bgPaint;
        private boolean swapColors;
        private boolean strikethrough;
        private Underline stdUnderline;
        private Underline imUnderline;
        
        DecorationImpl(Paint fgPaint, Paint bgPaint, boolean swapColors, boolean strikethrough, Underline stdUnderline, Underline imUnderline) {
            super(null);
            this.fgPaint = null;
            this.bgPaint = null;
            this.swapColors = false;
            this.strikethrough = false;
            this.stdUnderline = null;
            this.imUnderline = null;
            this.fgPaint = fgPaint;
            this.bgPaint = bgPaint;
            this.swapColors = swapColors;
            this.strikethrough = strikethrough;
            this.stdUnderline = stdUnderline;
            this.imUnderline = imUnderline;
        }
        
        private static boolean areEqual(Object o, Object obj) {
            if (o == null) {
                return obj == null;
            }
            return o.equals(obj);
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if (o == null) {
                return false;
            }
            Decoration.DecorationImpl decorationImpl;
            try {
                decorationImpl = (Decoration.DecorationImpl)o;
            }
            catch (ClassCastException ex) {
                return false;
            }
            return this.swapColors == decorationImpl.swapColors && this.strikethrough == decorationImpl.strikethrough && areEqual(this.stdUnderline, decorationImpl.stdUnderline) && areEqual(this.fgPaint, decorationImpl.fgPaint) && areEqual(this.bgPaint, decorationImpl.bgPaint) && areEqual(this.imUnderline, decorationImpl.imUnderline);
        }
        
        @Override
        public int hashCode() {
            int n = 1;
            if (this.strikethrough) {
                n |= 0x2;
            }
            if (this.swapColors) {
                n |= 0x4;
            }
            if (this.stdUnderline != null) {
                n += this.stdUnderline.hashCode();
            }
            return n;
        }
        
        private float getUnderlineMaxY(CoreMetrics coreMetrics) {
            float n = 0.0f;
            if (this.stdUnderline != null) {
                n = Math.max(n, coreMetrics.underlineOffset + this.stdUnderline.getLowerDrawLimit(coreMetrics.underlineThickness));
            }
            if (this.imUnderline != null) {
                n = Math.max(n, coreMetrics.underlineOffset + this.imUnderline.getLowerDrawLimit(coreMetrics.underlineThickness));
            }
            return n;
        }
        
        private void drawTextAndEmbellishments(Decoration.Label label, Graphics2D graphics2D, float x1, float n) {
            label.handleDraw(graphics2D, x1, n);
            if (!this.strikethrough && this.stdUnderline == null && this.imUnderline == null) {
                return;
            }
            float x2 = x1 + (float)label.getLogicalBounds().getWidth();
            CoreMetrics coreMetrics = label.getCoreMetrics();
            if (this.strikethrough) {
                Stroke stroke = graphics2D.getStroke();
                graphics2D.setStroke(new BasicStroke(coreMetrics.strikethroughThickness, 0, 0));
                float n2 = n + coreMetrics.strikethroughOffset;
                graphics2D.draw(new Line2D.Float(x1, n2, x2, n2));
                graphics2D.setStroke(stroke);
            }
            float underlineOffset = coreMetrics.underlineOffset;
            float underlineThickness = coreMetrics.underlineThickness;
            if (this.stdUnderline != null) {
                this.stdUnderline.drawUnderline(graphics2D, underlineThickness, x1, x2, n + underlineOffset);
            }
            if (this.imUnderline != null) {
                this.imUnderline.drawUnderline(graphics2D, underlineThickness, x1, x2, n + underlineOffset);
            }
            return;
        }
        
        @Override
        public void drawTextAndDecorations(Decoration.Label label, Graphics2D graphics2D, float n, float n2) {
            if (this.fgPaint == null && this.bgPaint == null && !this.swapColors) {
                this.drawTextAndEmbellishments(label, graphics2D, n, n2);
            }
            else {
                Paint paint = graphics2D.getPaint();
                Color paint2;
                Paint bgPaint;
                if (this.swapColors) {
                    bgPaint = ((this.fgPaint == null) ? paint : this.fgPaint);
                    if (this.bgPaint == null) {
                        if ((bgPaint instanceof Color)) {
                            Color color = (Color)bgPaint;
                            paint2 = ((33 * color.getRed() + 53 * color.getGreen() + 14 * color.getBlue() > 18500) ? Color.BLACK : Color.WHITE);
                        }
                        else {
                            paint2 = Color.WHITE;
                        }
                    }
                    else {
                        paint2 = this.bgPaint;
                    }
                }
                else {
                    paint2 = ((this.fgPaint == null) ? paint : this.fgPaint);
                    bgPaint = this.bgPaint;
                }
                if (bgPaint != null) {
                    Rectangle2D logicalBounds = label.getLogicalBounds();
                    Rectangle2D.Float o = new Rectangle2D.Float(n + (float)logicalBounds.getX(), n2 + (float)logicalBounds.getY(), (float)logicalBounds.getWidth(), (float)logicalBounds.getHeight());
                    graphics2D.setPaint(bgPaint);
                    graphics2D.fill(o);
                }
                graphics2D.setPaint(paint2);
                this.drawTextAndEmbellishments(label, graphics2D, n, n2);
                graphics2D.setPaint(paint);
            }
        }
        
        @Override
        public Rectangle2D getVisualBounds(Decoration.Label label) {
            Rectangle2D handleGetVisualBounds = label.handleGetVisualBounds();
            if (this.swapColors || this.bgPaint != null || this.strikethrough || this.stdUnderline != null || this.imUnderline != null) {
                float x = 0.0f;
                Rectangle2D logicalBounds = label.getLogicalBounds();
                float y = 0.0f;
                float a = 0.0f;
                if (this.swapColors || this.bgPaint != null) {
                    y = (float)logicalBounds.getY();
                    a = y + (float)logicalBounds.getHeight();
                }
                handleGetVisualBounds.add(new Rectangle2D.Float(x, y, (float)logicalBounds.getWidth(), Math.max(a, this.getUnderlineMaxY(label.getCoreMetrics())) - y));
                return handleGetVisualBounds;
            }
            else {
                return handleGetVisualBounds;
            }
        }
        
        @Override
        Shape getOutline(Decoration.Label label, float x1, float n) {
            if (!this.strikethrough && this.stdUnderline == null && this.imUnderline == null) {
                return label.handleGetOutline(x1, n);
            }
            CoreMetrics coreMetrics = label.getCoreMetrics();
            float underlineThickness = coreMetrics.underlineThickness;
            float underlineOffset = coreMetrics.underlineOffset;
            Rectangle2D logicalBounds = label.getLogicalBounds();
            float x2 = x1 + (float)logicalBounds.getWidth();
            Object s = null;
            if (this.stdUnderline != null) {
                s = new Area(this.stdUnderline.getUnderlineShape(underlineThickness, x1, x2, n + underlineOffset));
            }
            if (this.strikethrough) {
                BasicStroke o = new BasicStroke(coreMetrics.strikethroughThickness, 0, 0);
                float n2 = n + coreMetrics.strikethroughOffset;
                Area rhs = new Area(o.createStrokedShape(new Line2D.Float(x1, n2, x2, n2)));
                if (s == null) {
                    s = rhs;
                }
                else {
                    s.add(rhs);
                }
            }
            if (this.imUnderline != null) {
                Area rhs2 = new Area(this.imUnderline.getUnderlineShape(underlineThickness, x1, x2, n + underlineOffset));
                if (s == null) {
                    s = rhs2;
                }
                else {
                    s.add(rhs2);
                }
            }
            s.add(new Area(label.handleGetOutline(x1, n)));
            return new GeneralPath(s);
        }
        
        @Override
        public String toString() {
            StringBuffer sb = new StringBuffer();
            sb.append(super.toString());
            sb.append("[");
            if (this.fgPaint != null) {
                sb.append("fgPaint: " + this.fgPaint);
            }
            if (this.bgPaint != null) {
                sb.append(" bgPaint: " + this.bgPaint);
            }
            if (this.swapColors) {
                sb.append(" swapColors: true");
            }
            if (this.strikethrough) {
                sb.append(" strikethrough: true");
            }
            if (this.stdUnderline != null) {
                sb.append(" stdUnderline: " + this.stdUnderline);
            }
            if (this.imUnderline != null) {
                sb.append(" imUnderline: " + this.imUnderline);
            }
            sb.append("]");
            return sb.toString();
        }
    }
    
    public interface Label
    {
        public abstract CoreMetrics getCoreMetrics();
        
        public abstract Rectangle2D getLogicalBounds();
        
        public abstract void handleDraw(Graphics2D p0, float p1, float p2);
        
        public abstract Rectangle2D handleGetCharVisualBounds(int p0);
        
        public abstract Rectangle2D handleGetVisualBounds();
        
        public abstract Shape handleGetOutline(float p0, float p1);
    }
}
